;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of 1to1 Soccer: An Amstrad CPC Game
;;  Copyright (C) 2020 Utopia (@UtopiaRetroDev)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;-------------------------------------------------------------------------------

;; Include all CPCtelera constant definitions, macros and variables
.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "man/game.h.s"
.include "man/menu.h.s"
.include "man/character.h.s"
.include "man/character2.h.s"
.include "man/controls.h.s"
.include "man/credits.h.s"
.include "man/winner.h.s"
.include "sys/render.h.s"
.include "music_g.h.s"

.area _DATA
.area _CODE

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA ESPERAR ENTRE MENUS
;;

wait_menu::
   
   call  cpct_isAnyKeyPressed_f_asm
   jr nz, wait_menu                          ;;mientras alguna tecla este pulsada esperamos

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA LIMPIAR LA PANTALLA Y ESPERAR UN POCO ANTES DE PASAR AL SIGUIENTE
;;
  
clear_screen:

   cpctm_clearScreen_asm 0x00 ;;REPINTAMOS LA PANTALLA CON 0   

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA MANEJAR LAS INTERRUPCIONES
;;

int_cont:: .db 06

int_handler::
   push  af
   push  bc
   push  de
   push  hl

   ld    a, (int_cont)
   dec   a
   jr    nz, _cont

_zero:
   ;;de momento la musica esta para 50hz
   call cpct_akp_musicPlay_asm

   call  cpct_scanKeyboard_if_asm

   call  man_game_interrupt

   ld    a, #6             ;;volvemos a poner el contador a 6

_cont:
   ld    (int_cont), a     ;;guardamos el valor del contador

;.globl cpct_setPALColour_asm
   ;ld    h, a
   ;ld    l, #16
   ;call  cpct_setPALColour_asm

   pop   hl
   pop   de
   pop   bc
   pop   af

   ei
   reti

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA PONER EL NUEVO CONTROLADOR DE INTERRUPCIONES
;;

set_int_handler::
   call  cpct_waitVSYNC_asm
   halt
   halt
   call  cpct_waitVSYNC_asm

   ld hl, #0x38
   ld (hl), #0xC3
   inc hl
   ld (hl), #<int_handler
   inc hl
   ld (hl), #>int_handler
   ;inc hl
   ;ld (hl), #0xC9

   ld    a, #6             ;;ponemos el contador a 6
   ld    (int_cont), a     ;;guardamos el valor del contador

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; MAIN
;;

_main:: 
   call set_int_handler

   call sys_render_init

   ;;llamada para poner la musica
   ld  de, #_bobline
   call cpct_akp_musicInit_asm

   ;;llamada para el volumen
   ld a, #10                           ;;0 es el max y 16 el min
   call cpct_akp_setFadeVolume_asm

   ;;llamada para poner los SFX
   ld  de, #_sfx
   call cpct_akp_SFXInit_asm

game_restart::
   ;; INIT GAME ENTITIES
   call man_game_init

   ;; MENU INIT & RUN
   call man_menu_init

   call wait_menu

   call man_menu_run

   call clear_screen

   return_character_1::

   ;; SELECT CHARACTER & RUN
   call man_character_init

   call wait_menu

   call man_character_run

   call clear_screen

   ;; SELECT CHARACTER 2 & RUN
   call man_character_2_init

   call wait_menu

   call man_character_2_run

   call clear_screen


   ;; PLAY GAME
   call man_game_play

   jr nz, reset

   call clear_screen

   ;; WINNER MENU & RUN
   call man_winner_init
   call man_winner_run


   reset:
   call clear_screen

   ;; RESET GAME ENTITIES
   call man_game_restart

   jp game_restart
   

   ;; SELECT CONTROLS & RUN
   ;;parte del codigo correspondiente al menu de controles
   menu_controls::

   call clear_screen
   
   call man_controls_init

   call wait_menu

   call man_controls_run

   call clear_screen

   jp game_restart


   ;; SHOW CREDITS
   ;;parte del codigo correspondiente al menu de controles
   menu_credits::

   call clear_screen
   
   call man_credits_init

   call wait_menu

   call man_credits_run

   call clear_screen

   jp game_restart
   

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; EFECTOS ESPECIALES
;;
;; A = No Channel (0,1,2)
;; L = Instrument Number (>0)
;; H = Volume (0...F)
;; E = Note (0...143)
;; D = Speed (0 = As original, 1...255 = new Speed (1 is the fastest))
;; BC = Inverted Pitch (-#FFFF -> FFFF). 0 is no pitch. The higher the pitch, the lower the sound.

fuera_der::

    ld a, #4                  ;;bit mask  100
    ld l, #1
    ld h, #15
    ld e, #71
    ld d, #0
    ld bc, #-0xFFFF
    call cpct_akp_SFXPlay_asm


    ld a, #2                  ;;bit mask  100
    ld l, #1
    ld h, #15
    ld e, #61
    ld d, #0
    ld bc, #-0xFFFF
    call cpct_akp_SFXPlay_asm

ret


gol_der::


    ld a, #4                  ;;bit mask  100 solo derecha
    ld l, #1
    ld h, #15
    ld e, #91
    ld d, #0
    ld bc, #0xFFFF
    call cpct_akp_SFXPlay_asm

    ld a, #2                  ;;bit mask  100 solo derecha
    ld l, #1
    ld h, #15
    ld e, #101
    ld d, #0
    ld bc, #0xFFFF
    call cpct_akp_SFXPlay_asm

ret

fuera_izq::

    ld a, #1                  ;;bit mask  001 izq
    ld l, #1
    ld h, #15
    ld e, #71
    ld d, #0
    ld bc, #-0xFFFF
    call cpct_akp_SFXPlay_asm

    ld a, #2                  ;;bit mask  100
    ld l, #1
    ld h, #15
    ld e, #61
    ld d, #0
    ld bc, #-0xFFFF
    call cpct_akp_SFXPlay_asm


ret


gol_izq::

    ld a, #1                  ;;bit mask  001 solo izq
    ld l, #1
    ld h, #15
    ld e, #91
    ld d, #0
    ld bc, #0xFFFF
    call cpct_akp_SFXPlay_asm

    ld a, #2                  ;;bit mask  100 solo derecha
    ld l, #1
    ld h, #15
    ld e, #101
    ld d, #0
    ld bc, #0xFFFF
    call cpct_akp_SFXPlay_asm

ret

pitido_final::

    ld a, #2                  ;;bit mask  001 solo izq
    ld l, #2
    ld h, #15
    ld e, #70
    ld d, #0
    ld bc, #-0xFFFF
    call cpct_akp_SFXPlay_asm

ret
