;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of 1to1 Soccer: An Amstrad CPC Game
;;  Copyright (C) 2020 Utopia (@UtopiaRetroDev)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;-------------------------------------------------------------------------------

;;
;; ENTITY MANAGER
;;

.module Entity_Manager

.include "cpctelera.h.s"
.include "man/entity.h.s"
.include "cpct_functions.h.s"
.include "sys/physics.h.s"
.include "sys/render.h.s"
.include "src/sprites/sprites.h.s"
.include "comp/component.h.s"
.include "man/collision.h.s"
.include "man/ia_control.h.s"
.include "sys/scroll.h.s"
.include "src/sprites/precalculated.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; DATOS PRIVADOS
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


DefineComponentArrayStructure_Size _entity, max_entities, entity_size     
.db 0   ;;ponemos este aqui como trampita para que siempre haya un tipo invalido al final

ball_init_x = 119
player1_init_x = 97
player2_init_x = 135
floor_player = 152
floor_ball = 166
limit_right_ball = 230     
limit_right_player = 224;;la pantalla son 480 px - 18 px de los player - 16px de la porteria /2
vel_init = 0

e_cmp_player_input = e_cmp_default | e_cmp_input
e_cmp_ia_npc = e_cmp_default | e_cmp_ia
ball: DefineEntity e_type_ball, e_cmp_default, ball_init_x, floor_ball, limit_right_ball, vel_init, vel_init, 3, 10, 2, 3, draw_Sprite_Ball, #0x0000, 0 ;;jugadora por defecto de tipo 7: renderizable, movible y controlable por input
player_1: DefineEntity e_type_player1, e_cmp_player_input, player1_init_x, floor_player, limit_right_player, vel_init, vel_init, 8, 24, 3, 4, draw_Sprite_Boy_Right, draw_Sprite_Boy, 0  ;;jugador por defecto de tipo 7: renderizable, movible y controlable por input
player_2: DefineEntity e_type_player2, e_cmp_ia_npc, player2_init_x, floor_player, limit_right_player, vel_init, vel_init, 8, 24, 3, 4, draw_Sprite_Girl_Left, draw_Sprite_Girl, 0 ;;jugadora por defecto de tipo 7: renderizable, movible y controlable por input


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCIONES
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; PONER EN HL EL INICIO DEL ARRAY DE ENTIDADES
;; 

man_entity_getArrayHL::

    ld  hl, #_entity_array

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; PONER EN DE EL SPRITE DEL PRIMER JUGADOR
;; 

man_entity_getSprite_player1_DE::

    ld  ix, #_entity_array                  ;;guardamos la primera pos del array (pelota)

    ld  bc, #entity_size                    ;;meto en bc lo que ocupa una entidad
    add ix, bc                              ;;se lo sumo a ix para pasar a la siguiente (player 1)

    call man_entity_flip_right              ;;para que de igual como acabe que siempre salga igual

    ld  d, e_sprite_h(ix)
    ld  e, e_sprite_l(ix)

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; PONER EN DE EL SPRITE DEL SEGUNDO JUGADOR
;; 

man_entity_getSprite_player2_DE::

    ld  ix, #_entity_array                  ;;guardamos la primera pos del array (pelota)

    ld  bc, #entity_size                    ;;meto en bc lo que ocupa una entidad
    add ix, bc                              ;;se lo sumo a ix para pasar a la siguiente (player 1)
    add ix, bc                              ;;se lo sumo a ix para pasar a la siguiente (player 2)

    call man_entity_flip_left               ;;para que de igual como acabe que siempre salga igual

    ld  d, e_sprite_h(ix)
    ld  e, e_sprite_l(ix)

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; INICIALIZACION DEL MANAGER DE ENTIDADES
;; 

man_entity_init::
    ;;iniciamos los otros managers que dependen de el
    call man_collision_init

    ld  a, (_entity_num)      ;;ponemos el numero de entidades a 0
    xor a
    ld  (_entity_num), a

    ld  hl, #_entity_array      ;;ponemos el puntero de la ultima entidad a la primera posicion del array
    ld  (_entity_pend), hl

    ld  (hl), #e_type_invalid   ;;ponemos el primer elemento del array con tipo invalido

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; CREACION DE LAS ENTIDADES A USAR
;;      - dos jugadores, pelota y dos porterias
;; 

man_entity_generator::
    ld hl, #ball
    call man_entity_create  ;;creamos primero la pelota ya que es el unico que tenemos que ver las colisiones con el resto de entidades

    ld hl, #player_1
    call man_entity_create

    ld hl, #player_2
    call man_entity_create

    ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; CREAR UNA NUEVA ESTRELLA
;; input
;;     HL: puntero al array con los datos de inicializacion
;;
;; Precondicion:
;;      debe haber espacio disponible en el array de entidades
;;
;;ldir copia desde donde apunta hl a donde apunta de (el sitio donde los quiero copiar), tantos bytes como ponga en el registro bc

man_entity_create::               

    ld  de, (_entity_pend)
    ld  bc, #entity_size
    ldir

    ;;Collision
    ld  ix, (_entity_pend)
    ld  a, e_cmp(ix)        ;;guardamos en a su tipo de componente
    and #e_cmp_collider
    call nz, man_collision_add  ;;si son de tipo collision se meten en el array de collision

    ;;IA 
    ld  ix, (_entity_pend)
    ld  a, e_cmp(ix)        ;;guardamos en a su tipo de componente
    and #e_cmp_ia
    call nz, man_ia_add  ;;si son de tipo ia se meten en el array de ia
   
    ;;calculamos la dir de mem de los tiles y sus variables
    call man_entity_calculate_tile

    ;;PASAMOS A LA SIGUIENTE ENTIDAD
    ld  a, (_entity_num)    ;;aumentamos el numero de entidades
    inc a
    ld  (_entity_num), a

    ld   hl, (_entity_pend) ;;pasamos el puntero a la siguiente entidad
    ld   bc, #entity_size
    add  hl, bc
    ld   (_entity_pend), hl
    
    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; ACTUALIZAR ENTIDADES
;;  func_update_forall sirve para que esta funcion se pueda usar tanto para render como fisicas
;;  no usamos el matching ya que todas nuestras entidades tienen esos dos componentes
;;
;;  INPUT
;;      HL: direccion de memoria de la funcion que nos ha llamado. Es decir, actualiza fisicas o render de una entidad.
;;

man_entity_update_forall::
    ld  ix, #_entity_array      ;;cojo la primera entidad


    update_entitites:
    ld  a, e_type(ix)               ;;cojo su tipo
    ld  b, #e_type_invalid          ;;guardo en b el tipo de entidad invalido

    cp  b                           ;;comparo a con b
    jr  z, fin_update_entitites     ;;si da cero es que es invalida y hemos acabado de actualizar

func_update_forall == . + 1         ;;esta etiqueta nos permitira cambiar la funcion a la que llama el update, cambiando la direccion que hay en ella
    call  #0x4000                   ;;ponemos una direccion aleatoria, ya que se cambiara el codigo con la etiqueta de arriba

    ld  bc, #entity_size            ;;meto en bc lo que ocupa una entidad
    add ix, bc                      ;;se lo sumo a ix para pasar a la siguiente
    jr  update_entitites            ;;volvemos a repetir el proceso hasta que se salga por una invalida

    fin_update_entitites:
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; RESETEAMOS LAS POSICIONES Y VELOCIDADES DE LAS ENTIDADES CUANDO ES GOL
;;      

man_entity_reset_goal::

    ld  ix, #_entity_array      ;;guardamos la primera pos del array (pelota)
    ld  e_x(ix), #ball_init_x   ;;reiniciamos su posicion y vel
    ld  e_y(ix), #floor_ball
    ld  e_vx(ix), #vel_init
    ld  e_vy(ix), #vel_init
    call man_entity_calculate_tile

    ld  bc, #entity_size        ;;meto en bc lo que ocupa una entidad
    add ix, bc                  ;;se lo sumo a ix para pasar a la siguiente (player 1)

    ld  e_x(ix), #player1_init_x   ;;reiniciamos su posicion y vel
    ld  e_y(ix), #floor_player
    ld  e_vx(ix), #vel_init
    ld  e_vy(ix), #vel_init
    call man_entity_flip_right
    call man_entity_calculate_tile

    ld  bc, #entity_size        ;;meto en bc lo que ocupa una entidad
    add ix, bc                  ;;se lo sumo a ix para pasar a la siguiente (player 2)

    ld  e_x(ix), #player2_init_x   ;;reiniciamos su posicion y vel
    ld  e_y(ix), #floor_player
    ld  e_vx(ix), #vel_init
    ld  e_vy(ix), #vel_init
    call man_entity_flip_left
    call man_entity_calculate_tile

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; RESETEAMOS LAS POSICIONES Y VELOCIDADES DE LAS ENTIDADES CUANDO ES OUT LEFT
;;      


man_entity_reset_out_left::
    ld  ix, #_entity_array      ;;guardamos la primera pos del array (pelota)
    ld  e_x(ix), #32            ;;reiniciamos su posicion y vel
    ld  e_y(ix), #floor_ball
    ld  e_vx(ix), #vel_init
    ld  e_vy(ix), #vel_init
    call man_entity_calculate_tile

    ld  bc, #entity_size        ;;meto en bc lo que ocupa una entidad
    add ix, bc                  ;;se lo sumo a ix para pasar a la siguiente (player 1)

    ld  e_x(ix), #20            ;;reiniciamos su posicion y vel
    ld  e_y(ix), #floor_player
    ld  e_vx(ix), #vel_init
    ld  e_vy(ix), #vel_init
    call man_entity_flip_right
    call man_entity_calculate_tile

    ld  bc, #entity_size        ;;meto en bc lo que ocupa una entidad
    add ix, bc                  ;;se lo sumo a ix para pasar a la siguiente (player 2)

    ld  e_x(ix), #player2_init_x   ;;reiniciamos su posicion y vel
    ld  e_y(ix), #floor_player
    ld  e_vx(ix), #vel_init
    ld  e_vy(ix), #vel_init
    call man_entity_flip_left
    call man_entity_calculate_tile

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; RESETEAMOS LAS POSICIONES Y VELOCIDADES DE LAS ENTIDADES CUANDO ES OUT RIGHT
;;      


man_entity_reset_out_right::
    ld  ix, #_entity_array      ;;guardamos la primera pos del array (pelota)
    ld  e_x(ix), #205            ;;reiniciamos su posicion y vel
    ld  e_y(ix), #floor_ball
    ld  e_vx(ix), #vel_init
    ld  e_vy(ix), #vel_init
    call man_entity_calculate_tile

    ld  bc, #entity_size        ;;meto en bc lo que ocupa una entidad
    add ix, bc                  ;;se lo sumo a ix para pasar a la siguiente (player 1)

    ld  e_x(ix), #player1_init_x            ;;reiniciamos su posicion y vel
    ld  e_y(ix), #floor_player
    ld  e_vx(ix), #vel_init
    ld  e_vy(ix), #vel_init
    call man_entity_flip_right
    call man_entity_calculate_tile

    ld  bc, #entity_size        ;;meto en bc lo que ocupa una entidad
    add ix, bc                  ;;se lo sumo a ix para pasar a la siguiente (player 2)

    ld  e_x(ix), #212   ;;reiniciamos su posicion y vel
    ld  e_y(ix), #floor_player
    ld  e_vx(ix), #vel_init
    ld  e_vy(ix), #vel_init
    call man_entity_flip_left
    call man_entity_calculate_tile

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion para cambiar el sprite y hacer flip
;;
;; INPUT:
;;      IX: puntero a la entidad a actualizar.
;;   

man_entity_flip::

    ld  a, (hl)
    ld  e_sprite_l(ix), a
    inc hl
    ld  a, (hl)
    ld  e_sprite_h(ix), a

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion para coger la dir de mem del sprite cuando va hacia la izq
;;
;; INPUT:
;;      IX: puntero a la entidad a actualizar.
;;   

man_entity_flip_left::

    ld  h, e_spriteArray_h(ix)
    ld  l, e_spriteArray_l(ix)
    inc hl
    inc hl

    call man_entity_flip

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion para coger la dir de mem del sprite cuando va hacia la derecha
;;
;; INPUT:
;;      IX: puntero a la entidad a actualizar.
;;   

man_entity_flip_right::

    ld  h, e_spriteArray_h(ix)
    ld  l, e_spriteArray_l(ix)

    call man_entity_flip

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; CAMBIAMOS EL SPRITE DE PLAYER 1
;;      INPUT
;;          HL: dir de mem del sprite
;;          DE: dir de mem del array con los sprites
;;

man_entity_change_character_1::

    ld  ix, #_entity_array                  ;;guardamos la primera pos del array (pelota)

    ld  bc, #entity_size                    ;;meto en bc lo que ocupa una entidad
    add ix, bc                              ;;se lo sumo a ix para pasar a la siguiente (player 1)

    ld  e_sprite_l(ix), l
    ld  e_sprite_h(ix), h

    ex  de, hl
    ld  e_spriteArray_l(ix), l
    ld  e_spriteArray_h(ix), h

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; CAMBIAMOS EL SPRITE DE PLAYER 2
;;      INPUT
;;          HL: dir de mem del sprite
;;          DE: dir de mem del array con los sprites
;;

man_entity_change_character_2::

    ld  ix, #_entity_array                  ;;guardamos la primera pos del array (pelota)

    ld  bc, #entity_size                    ;;meto en bc lo que ocupa una entidad
    add ix, bc                              ;;se lo sumo a ix para pasar a la siguiente (player 1)
    add ix, bc                              ;;se lo sumo a ix para pasar a la siguiente (player 2)

    ld  e_sprite_l(ix), l
    ld  e_sprite_h(ix), h

    ex  de, hl
    ld  e_spriteArray_l(ix), l
    ld  e_spriteArray_h(ix), h

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; QUITAMOS LA IA DEL JUGADOR 2
;;   

man_entity_change_ia::
    
    ld  ix, #_entity_array      ;;guardamos la primera pos del array (pelota)

    ld  bc, #entity_size        ;;meto en bc lo que ocupa una entidad
    add ix, bc                  ;;se lo sumo a ix para pasar a la siguiente (player 1)
    add ix, bc                  ;;se lo sumo a ix para pasar a la siguiente (player 2)

    ld  e_cmp(ix), #e_cmp_player_input    ;;ponemos que vaya con input

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; calculamos la dir de mem del tile de la esq superior izq
;;  INPUT:
;;      IX: entidad   
;;

man_entity_calculate_tile::

    ;;A = y
    ld   a, e_y(ix)
    ;;A = ty (y/8)
    srl  a ;; |
    srl  a ;; | A = A/8
    srl  a ;; |
    ;; HL = A (HL = ty)
    ld   h, #0
    ld   l, a
    ;; HL = 60*HL
    add  hl, hl    ;; HL = ty*2
    add  hl, hl    ;; HL = ty*4
    push hl        ;;guardamos el *4
    add  hl, hl    ;; HL = ty*8
    push hl        ;;guardamos el ty*8
    add  hl, hl    ;; HL = ty*16
    ld   d, h ;;   | guardamos el ty*16 
    ld   e, l ;;   |
    add  hl, hl    ;; HL = ty*32
    add  hl, de    ;; HL = ty*48
    pop  de        ;;recuperamos en DE el *8
    add  hl, de    ;; HL = ty*56
    pop  de        ;;recuperamos en DE el *4
    add  hl, de    ;; HL = ty*60

    ;;A = x
    ld   a, e_x(ix)
    ;;A = tx (x/4)
    srl  a ;; | A = A/4
    srl  a ;; |

    add_hl_a       ;;HL = ty * tw + tx
    ld   de, #_g_tilemap
    add  hl, de

    ld  e_tilePos_l(ix), l      ;;actualizamos la posicion anterior del tilemap
    ld  e_tilePos_h(ix), h   

    ;;el tx no va a ser realmente en tiles, va a ser en el caso de tx por ejemplo 4*((int)x/4)
    ld   a, e_y(ix)
    and  #0xF8
    ld   e_ty(ix), a
    ld   a, e_x(ix)
    and  #0xFC
    ld   e_tx(ix), a

    ret