;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of 1to1 Soccer: An Amstrad CPC Game
;;  Copyright (C) 2020 Utopia (@UtopiaRetroDev)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;-------------------------------------------------------------------------------

;;
;; GAME MANAGER
;;
.module Game_Manager

.include "cpctelera.h.s"
.include "sys/render.h.s"
.include "sys/physics.h.s"
.include "man/entity.h.s"
.include "cpct_functions.h.s"
.include "sys/collision.h.s"
.include "man/collision.h.s"
.include "sys/ia_control.h.s"
.include "man/ia_control.h.s"
.include "sys/scroll.h.s"
.include "man/scroll.h.s"
.include "man/game.h.s"
.include "src/sprites/menus/menu_goal_z.h.s"
.include "src/sprites/menus/menu_out_z.h.s"
.include "music_g.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  VARIABLES DEL JUEGO
;;

mode_game::     .db 0
goal_player1::  .db 0
goal_player2::  .db 0
reset_goal::    .db 0
;;luego habra un reset out
winner::        .db 0
num_interrupt:: .db 50      ;;hay 300 por s/ 50 por it
time_game::     .dw #0x0200 ;;dos minutos de partida
time_change::   .db 0       ;;para que solo repinte el tiempo si ha cambiado
restore_background:: .db 0  ;;si esta a 0 es que hay que repintarlo
continue_match::     .db 0  ;;valor del contador en ese momento
delay_ia::           .db 16  ;;valor para que cuando sea saque de centro la ia tarde en empezar

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; INICIALIZACION DEL JUEGO
;; 

man_game_init::

    ;;Init sistems
    call man_entity_init
    call man_collision_init
    call man_ia_init
    call sys_collision_init
    call sys_ia_control_init
    call man_entity_generator
    
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA ESPERAR A ESA INTERRUP
;; 

man_game_wait::

    loop_wait:
    ld  a, (int_cont)
    cp  #1
    jr  nz, loop_wait

    ret

man_game_wait_continue_match::

    loop_wait_match:
    ld  a, (continue_match)
    ld  b, a
    ld  a, (int_cont)
    cp  b
    jr  nz, loop_wait_match

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; BUCLE DEL JUEGO
;; 

man_game_play::
    
    ;;ponemos el tiempo y el num de interrupciones a su valor inicial
    ld  a, #50
    ld  (num_interrupt), a
    ld  hl, #0x0200
    ld  (time_game), hl
    
    call man_scroll_init
    call set_int_handler
    call sys_render_scores_time_init

    loop:

        call man_game_check_clock   ;;para que suene si es el ultimo segundo
        
        call man_game_check_finish
        ret  z                      ;;si vuelve con 0 es que ha pasado el tiempo 

        xor  a 
        ld   (restore_background), a

        ld  a, (mode_game)
        dec a
        jr  nz, multiplayer

        ld  a, (delay_ia)
        cp  #0
        jr  z, init_ia

        ld  c, a
        ld  a, (int_cont)
        ld  (continue_match), a

        call man_game_wait_continue_match
        dec  c 
        ld  a, c
        ld  (delay_ia), a                       ;;actualizamos su valor
        jr  multiplayer

        init_ia:
        call sys_ia_update       ;; solo llamara si el jugador ha puesto singleplayer (es decir si esta en el modo 1, ya que 1-1=0)
        
        multiplayer:
        call sys_physics_update
        call sys_collision_update   
        call man_game_check_goal
        jr  nz, loop

        call sys_scroll_update      ;;hacemos los calculos para el scroll
        
        call man_game_wait

        call sys_scroll_draw
        call sys_render_scores_time      ;;pintamos las puntuaciones en la pantalla
        
        call cpct_waitVSYNC_asm

        ld   a, (restore_background)
        or   a
        jr   nz, no_restore 

        call sys_scroll_restore_background

        no_restore:
        call sys_render_update

        call sys_scroll_recalculate_tile

        ld  hl, #Key_Esc
        call cpct_isKeyPressed_asm
        ret nz                      ;;si vuelve con 0 es que se ha salido de la partida

    jr loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; REINICIAMOS EL JUEGO
;; 

man_game_restart::
    ;;Iniciamos los datos a 0
    xor a                       ;;ponemos A a 0
    ld  (goal_player1), a
    ld  (goal_player2), a
    ld  (reset_goal), a
    ld  (winner), a

    call man_scroll_reset

    ret
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; COMPROBAMOS SI HAY QUE REINICIAR POSICIONES PORQUE SE HA METIDO GOL
;; 

man_game_check_goal:

    ld  a, (reset_goal) ;;miramos a ver si se ha activado el byte para resetear posiciones
    or  a               ;;lo comparamos consigo misma para saber si es 0
    ret z               ;;si es 0 es que no hay que resetear


    ld   l, #0
    call cpct_setVideoMemoryOffset_asm  ;;cambiamos el offset de la memoria (donde empieza lo que se ve)

    ld  a, (reset_goal)
    cp  #2
    jr  c, goal
    jr  z, out_left
    jr  nc, out_right

    goal:

    call man_entity_reset_goal      ;;resetea las posiciones de los jugadores
    xor a                           ;;pone A a 0
    ld (reset_goal), a              ;;volvemos a poner el reset en 0 para que no resetee todo el rato

    call man_scroll_reset

    ;;ponemos un random a la ia entre 0 y 9 para que cada vez que meta gol empiece con un valor random
    call cpct_getRandom_mxor_u8_asm
    ld  d, l
    ld  a, #0x10
    push af
    inc sp
    push de
    inc sp
    call __moduchar
    pop af  
    ld  a, l
    ld  (delay_ia), a

    ld  hl, #_menu_goal_z_end           ;;ponemos la pantalla de gol

    jr  fin_check_goal

    out_left:
    call man_entity_reset_out_left      ;;resetea las posiciones de los jugadores
    xor a                               ;;pone A a 0
    ld (reset_goal), a                  ;;volvemos a poner el reset en 0 para que no resetee todo el rato
    
    call man_scroll_reset               ;;se llama desde aqui y no en comun porque el init pone en 40 el valor de init_scroll

    xor a
    ld  (init_scroll), a                ;;desde la izquierda el scroll es 0

    ld  hl, #_menu_out_z_end            ;;ponemos la pantalla de out
 
    jr  fin_check_goal

    out_right:
    call man_entity_reset_out_right      ;;resetea las posiciones de los jugadores
    xor a                               ;;pone A a 0
    ld (reset_goal), a                  ;;volvemos a poner el reset en 0 para que no resetee todo el rato
    
    call man_scroll_reset

    ld  a, #16                          ;;le ponemos un retraso a la ia cuando saca de su campo para que no salga de golpe
    ld  (delay_ia), a

    ld  a, #80
    ld  (init_scroll), a                ;;desde la izquierda el scroll es 0
    ld  hl, #_menu_out_z_end            ;;ponemos la pantalla de out

    fin_check_goal:

    ld  de, #0xFFFF 
    call cpct_zx7b_decrunch_s_asm

    ld  a, (int_cont)
    ld  (continue_match), a

    .rept 40
        call man_game_wait_continue_match
        halt
    .endm

    ld  a, #1
    ld  (restore_background), a

    call man_scroll_init                        ;;reseteamos el scroll
    call sys_render_scores_time_init            ;;pintamos las puntuaciones en la pantalla 

    call set_int_handler

    add #1                                      ;;si le agregamos uno a A nos asguramos que siempre vuelva con nz       

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; COMPROBAMOS SI ALGUNO HA METIDO YA LOS GOLES MAX Y ACABA LA PARTIDA
;; 

man_game_check_finish:


    ld  a, (time_game+1)
    ld  h, a
    ld  a, (time_game)
    ld  l, a

    ;;en a ya tenemos metida L
    ;;miramos si HL es 0000
    or  h

    ret nz

    ;;si han metido gol se comprueba si alguno ha llegado a 7
    ld  a, (goal_player2)
    ld  b, a
    ld  a, (goal_player1)
    cp  b
    jr  z, draw
    jr  nc, winner_player1
    jr  c, winner_player2


    draw:
    ld  a, #0
    ld  (winner), a

    xor a               ;;para que vuelva con 0
    ret

    winner_player1:
    ld  a, #1
    ld  (winner), a     

    xor a               ;;para que vuelva con 0
    ret
    
    winner_player2:
    ld  a, #2
    ld  (winner), a

    xor a               ;;para que vuelva con 0
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA MANEJAR LAS INTERRUPCIONES DURANTE LA PARTIDA
;; 

man_game_interrupt::

    ld  a, (num_interrupt)      ;;incrementamos el numero de interrupciones en 1
    dec a

    ld  b, a                    ;;me lo guardo para no perderlo

    or  a
    jr  nz, continue_inc        ;;si con la resta no se produce acarreo es que hl es mayor

    call man_game_time          ;;hago que el tiempo baje un segundo
    ld  a, #50                  ;;si ha llegado a 300 lo volvemos a poner en 0
    
    ld  (num_interrupt), a      ;;cambio el valor de el num de interrupciones
   
    ret

    continue_inc:
    ld  a, b
    ld  (num_interrupt), a      ;;actualizamos el valor del num de interrupciones

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA RESTAR UNO AL TIEMPO
;; 

man_game_time::

    ld  a, #1
    ld  (time_change), a

    ld  a, (time_game)
    dec a
    daa
    ld  (time_game), a
    ret  nc                 ;;si no hay c no hace falta que actualice la H

    ld  a, #0x0059
    ld  (time_game), a      ;;si ha dado c L pasaria a 99 y nosotros queremos que sea 59 para poner formato mm:ss

    ld  a, (time_game+1)
    ld  b, #0
    sbc b
    daa
    ld  (time_game+1), a    

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION COMPROBAR RELOJ Y SI QUEDAN 3 SEG QUE SUENE
;; 
man_game_check_clock::

    ld  a, (time_game+1)    ;; miramos la hora
    or a                    ;; si es cero
    ret nz

    ld  a, (time_game)      ;; miramos que son los ultimos 3 seg
    ld  b, #1
    cp  b
    call z, pitido_final

ret