;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of 1to1 Soccer: An Amstrad CPC Game
;;  Copyright (C) 2020 Utopia (@UtopiaRetroDev)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;-------------------------------------------------------------------------------

;;
;; SCROLL MANAGER
;;

.module Scroll_Manager

.include "cpctelera.h.s"
.include "sys/render.h.s"
.include "cpct_functions.h.s"
.include "man/scroll.h.s"
.include "src/sprites/sprites.h.s"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; DATOS PRIVADOS
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;tile negro
_black_tile::
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0
	.db #0x00      ;0

;;creamos la estructura del scroll
hwscroll::      DefineScroll #0xC000, _g_tilemap, #0
column_scroll:: .db 0
scroll_offset:: .db 0
tiles_to_scroll:: .db 0
init_scroll::   .db 40   

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCIONES
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION DE INICIALIZACION DEL SCROLL
;;

man_scroll_init::

    ;;Ponemos el tileset
    ld   c, #scr_tile_width
    ld   b, #map_height
    ld   de, #map_width
    ld   hl, #_tiles_00
    call cpct_etm_setDrawTilemap4x8_ag_asm

    ;;ponemos la pantalla en medio
    ld   ix, #hwscroll
    ld   a, (init_scroll)
    sla  a                       ;;multiplico por 2
    ld   e, hs_pVideo_l(ix)      
    ld   d, hs_pVideo_h(ix)
    add_de_a
    ld   hs_pVideo_l(ix), e
    ld   hs_pVideo_h(ix), d 

    ld   e, hs_pTilemap_l(ix)    
    ld   d, hs_pTilemap_h(ix)
    ld   a, (init_scroll)         ;; add_de_a se carga el valor de a
    srl  a                       ;;divido entre dos
    add_de_a
    ld   hs_pTilemap_l(ix), e
    ld   hs_pTilemap_h(ix), d

    ld   a, (init_scroll)
    ld   hs_scroll(ix), a
    ld   l, hs_scroll(ix)
    call cpct_setVideoMemoryOffset_asm  ;;cambiamos el offset de la memoria (donde empieza lo que se ve)

    ;;pintamos el tileset por primera vez pero en la mitad del campo
    ld   e, hs_pTilemap_l(ix)    
    ld   d, hs_pTilemap_h(ix)      ;; DE = puntero al tilemap
    ld   l, hs_pVideo_l(ix)      
    ld   h, hs_pVideo_h(ix)        ;; HL = puntero a la memoria de video 

    call cpct_etm_drawTilemap4x8_ag_asm

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION DE RESETEAR LOS DATOS NECESARIOS PARA EL SCROLL
;;

man_scroll_reset::

    ld   ix, #hwscroll

    ld   hs_pVideo_l(ix), #0x00
    ld   hs_pVideo_h(ix), #0xC0

    ld   hs_scroll(ix), #0

    ld   hl, #_g_tilemap
    ld   hs_pTilemap_l(ix), l
    ld   hs_pTilemap_h(ix), h

    xor a                       ;;la ponemos a 0
    ld  (scroll_offset), a
    ld  (tiles_to_scroll), a
    ld  (column_scroll), a

    ld   a, #40
    ld   (init_scroll), a

    ;ld   l, #0

    ;call cpct_setVideoMemoryOffset_asm  ;;cambiamos el offset de la memoria (donde empieza lo que se ve) 

    cpctm_clearScreen_asm 0x00  ;;REPINTAMOS LA PANTALLA CON 0

    ret