;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of 1to1 Soccer: An Amstrad CPC Game
;;  Copyright (C) 2020 Utopia (@UtopiaRetroDev)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;-------------------------------------------------------------------------------

;;
;;  COLLISION SYSTEM
;;

.include "man/collision.h.s"
.include "cpctelera.h.s"
.include "man/entity.h.s"
.include "sys/physics.h.s"

.module Collision_System

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; INICIALIZACION DEL SISTEMA DE COLISION
;; 

sys_collision_init::

    call man_collision_getArrayHL
    ld  (_ent_array_ptr), hl
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; ACTUALIZACION DEL SISTEMA DE COLISION
;; 

sys_collision_update::

    _ent_array_ptr = . + 1
    ld  hl, #0x0000

    ld  e, (hl)         ;;meto en DE la dir de mem de la primera entidad (LA PELOTA)
    inc hl
    ld  d, (hl)
    inc hl

    ld__ixl_e           ;;guardamos DE en IX
    ld__ixh_d


_next_iy:
    ld  e, (hl)         ;;meto en DE la dir de mem almacenada en hl (los jugadores)
    inc hl
    ld  d, (hl)
    inc hl

    ld  a, e            ;;comprobamos que DE no sea nula (=! 0x0000)
    or  d
    ret z

    ld__iyl_e           ;;guardamos DE en IY
    ld__iyh_d

    ;;comprobamos las colisiones entre las entidades IX e IY
    call sys_collision_check


    jr _next_iy            ;;seguimos con la siguiente entidad



;;hemos dividido el jugador en bounding boxes de 6px de ancho y 8px de alto
box_w = 5
box_h = 3

ball_limit = 2

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; COMPROBACION DE COLISION ENTRE DOS ENTIDADES
;;      INPUT
;;          IX: pelota
;;          IY: jugador
;; 

sys_collision_check::
    ;;PRIMERO COMPROBAMOS X PORQUE ES MENOS FRECUENTE

    ;; if ( A < B )
    ;; if ( e_x(ix) + e_w(ix) < e_x(iy) ) __no_collision
    ;; if ( e_x(ix) + e_w(ix) - e_x(iy) < 0 ) __no_collision
    ;; CASO DE DARLE A LA PELOTA HACIA LA IZQ
    ld  a, e_x(ix)
    add e_w(ix)
    sub e_x(iy)
    ret c           ;;si es carry es que no hay colision

    ;; if ( C < D )
    ;; if ( e_x(iy) + e_w(iy) < e_x(ix) ) __no_collision
    ;; if ( e_x(iy) + e_w(iy) - e_x(ix) < 0 ) __no_collision

    ld  a, e_x(iy)
    add e_w(iy)
    sub e_x(ix)
    ret c


    ;; if ( e_y(ix) + e_h(ix) < e_y(iy) ) __no_collision
    ;; if ( e_y(ix) + e_h(ix) - e_y(iy) < 0 ) __no_collision
    ;; si no hay colision ya nos podemos salir de la funcion
    ld  a, e_y(ix)
    add e_h(ix)
    sub e_y(iy)
    ret c                  

    ;; if ( e_y(iy) + e_h(iy) < e_y(ix) ) __no_collision
    ;; if ( e_y(iy) + e_h(iy) - e_y(ix) < 0 ) __no_collision

    ld  a, e_y(iy)
    add e_h(iy)
    sub e_y(ix)
    ret c


    ;;comprobamos si le esta dando en mitad de la cabeza o en los pies
    ld  a, e_cmp(iy)        ;;guardamos en a su tipo de componente
    and #e_cmp_ia
    jr nz, no_head                  ;;si es ia que no se le pueda quedar en la cabeza  

    ;;cabeza
    call sys_collision_check_head
    ret nc

    ;;pies
    no_head:
    call sys_collision_check_feet
    ret nc

    ;;if ( B < D ) 
    ;;hacemos esta comprobacion para ver le esta dando por un lado o por otro
    ld  a, e_x(iy)
    sub e_x(ix)
    jr c, col_2
    
    ld  e_vx(ix), #-1

    jr  change_vy

    col_2:
    ld  e_vx(ix), #1


    change_vy:          

    ld  a, #10
    ld  (ball_time), a          ;;se pone un valor para que se pare la pelota cuando llegue a 0 en el physics

    ld  a, e_y(iy)              ;;hacemos la comprobacion para ver si le da en la cabeza o en el cuerpo
    add #box_h
    sub e_y(ix)
    jr  c, hit_down

    ld  e_vy(ix), #-10          ;;si le da con la cabeza se le pone velocidad en y mayor
    ld  a, #-10
    ld  (ball_jump), a          ;;para que despues en physics haga rebotes (NUMEROS PARES)
    ret 

    hit_down:
    ld  e_vy(ix), #-4           ;;si le da con el cuerpo la velocidad en y en menor 
    ld  a, #-4          
    ld  (ball_jump), a          ;;para que despues en physics haga rebotes (NUMEROS PARES)

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; COMPROBACION DE COLISION ENTRE LA PELOTA Y LA CABEZA DEL JUGADOR
;;      INPUT
;;          IX: pelota
;;          IY: jugador
;; 

sys_collision_check_head:: 

    ;; if ( A < B )
    ;; if ( e_x(ix) + 2*ball_limit < e_x(iy) + box_w ) __no_collision
    ;; if ( e_x(ix) + 2*ball_limit - e_x(iy) - box_w < 0 ) __no_collision
    ;; CASO DE DARLE A LA PELOTA HACIA LA IZQ
    ld  a, e_x(ix)
    add #(2*ball_limit)
    sub #box_w
    sub e_x(iy)
    ret c           ;;si es carry es que no hay colision

    ;; if ( C < D )
    ;; if ( e_x(iy) + box_w < e_x(ix) + ball_limit ) __no_collision
    ;; if ( e_x(iy) + box_w - e_x(ix) - ball_limit < 0 ) __no_collision

    ld  a, e_x(iy)
    add #box_w
    sub #ball_limit
    sub e_x(ix)
    ret c


    ;; if ( e_y(ix) + e_h(ix) < e_y(iy) ) __no_collision
    ;; if ( e_y(ix) + e_h(ix) - e_y(iy) < 0 ) __no_collision
    ld  a, e_y(ix)
    add e_h(ix)
    sub e_y(iy)
    ret c                  

    ;; if ( e_y(iy) + box_h < e_y(ix) ) __no_collision
    ;; if ( e_y(iy) + box_h - e_y(ix) < 0 ) __no_collision

    ld  a, e_y(iy)
    add #box_h
    sub e_y(ix)
    ret c

    ld  e_vx(ix), #0            ;;si le da con el centro de la cabeza se mueve en vertical solo
    ld  e_vy(ix), #-10

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; COMPROBACION DE COLISION ENTRE LA PELOTA Y LOS PIES DEL JUGADOR
;;      INPUT
;;          IX: pelota
;;          IY: jugador
;; 

sys_collision_check_feet::
    ;; if ( A < B )
    ;; if ( e_x(ix) + e_w(ix) < e_x(iy) + box_w ) __no_collision
    ;; if ( e_x(ix) + e_w(ix) - e_x(iy) - box_w < 0 ) __no_collision
    ;; CASO DE DARLE A LA PELOTA HACIA LA IZQ
    ld  a, e_x(ix)
    add e_w(ix)
    sub #4
    sub e_x(iy)
    ret c           ;;si es carry es que no hay colision

    ;; if ( C < D )
    ;; if ( e_x(iy) + box_w < e_x(ix) ) __no_collision
    ;; if ( e_x(iy) + box_w - e_x(ix) < 0 ) __no_collision

    ld  a, e_x(iy)
    add #4
    sub e_x(ix)
    ret c
    
    ;; if ( e_y(ix) + 2*ball_limit < e_y(iy) + e_h(iy) ) __no_collision
    ;; if ( e_y(ix) + 2*ball_limit - e_y(iy) - e_h(iy) < 0 ) __no_collision
    ld  a, e_y(ix)
    add #(1*ball_limit)
    sub e_y(iy)
    ret c           ;;a veces da carry aqui y luego con el siguiente ya no
    sub e_h(iy)
    ret c  

    ;; if ( C < D )
    ;; if ( e_y(iy) + e_h(iy) < e_y(ix) ) __no_collision
    ;; if ( e_y(iy) + e_h(iy) - e_y(ix) < 0 ) __no_collision

    ld  a, e_y(iy)
    add e_h(iy)
    sub e_y(ix)
    ret c

    ld  e_vx(ix), #0    ;;para la pelota si le ha dado con los pies
    ld  e_vy(iy), #-6   ;;el jugador rebota al darle con los pies

    ret