;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of 1to1 Soccer: An Amstrad CPC Game
;;  Copyright (C) 2020 Utopia (@UtopiaRetroDev)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;-------------------------------------------------------------------------------

;;
;; IA control system
;;
.module sys_ia_control

.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "man/entity.h.s"
.include "man/ia_control.h.s"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; DATOS 
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ball_last_x:: .db 0
ball_last_y:: .db 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; INICIMOS EL SYS METIENDO EL ARRAY DE IA EN HL
;;
;;  Inits AI system
;;  INPUT:
;;  IX = puntero al array de entidades
;;
;; Unicamente el init es para guardarnos el puntero en ia 

sys_ia_control_init::
    xor a
    ld  (ball_last_x), a
    ld  (ball_last_y), a

    call man_ia_getArrayHL
    ld  (_ent_array_ptr), hl
    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; UPDATE DEL ARRAY DE IA (Solo el jugador de ia realmente)
;;
;; Update todas las entidades con ia
;;  INPUT:
;;  IX = puntero al entity array
;;  A = numero de elementos en el array
;;

sys_ia_update::
_ent_array_ptr = . + 1
    ld  hl, #0x0000

    ld  e, (hl)         ;;meto en DE la dir de mem de la primera entidad (LA IA)
    inc hl
    ld  d, (hl)
    inc hl

    ld__ixl_e           ;;guardamos DE en IX
    ld__ixh_d


    ;; guardamos en IY la pelota
    call man_entity_getArrayHL

    ex de, hl

    ld__iyl_e           ;;guardamos DE en IY
    ld__iyh_d

    
    ;;dependiendo del nivel de la ia actuara antes o mas tarde
    ld a, (ia_level_man)            ;;cogemos ia_level y lo vamos reduciendo como contador cuando sea 0 actuara
    dec a
    ld (ia_level_man), a
    jr  nz, move_ia

    ld a, e_x(iy)
    ld (ball_last_x), a
    ld a, e_y(iy)
    ld (ball_last_y), a

    ld a, e_ia_level(ix)
    ld (ia_level_man), a
    
    move_ia:
    call sys_ia_move                ;; si ia_level es 0 se movera

ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; METODO PARA ELEGIR QUE MOVIMIENTO DEBE HACER NUESTRA IA
;;
;;  INPUT:
;;  IX = puntero personaje ia
;;  IY = puntero al balon
;; Veremos donde esta la pelota y se dirigira hacia el lugar
;; cuando llegue a dicho lugar se parara
sys_ia_move::

    ld  a, (ball_last_x)
    ld  b, a
    ld  a, e_x(ix)
    cp b
    jr nc, move_left        
    ;;Si pasa aqui la pelota estara en la derecha y debera ver cuando saltarla
    ;;primero vemos is el jugador esta encima de la pelota para que se baje a la derecha
    ld  a, (ball_last_y)
    ld  b, a
    ld  a, e_y(ix)
    add e_h(ix)
    cp  b              
    jr  nc, ball_on_floor
    ;; si la pelota esta por debajo del jugador se mueve si o si a la derecha
    call sys_ia_move_right
    ret

    ball_on_floor:
    ld  a, (ball_last_x)
    ld  b, a
    ld  a, e_x(ix)
    add e_w(ix)
    add #3                      ;;MODIFICAR PARA RECUPERAR PELOTA
    cp  b           
    jr  c, only_move_right      ;;cuando la x de la ia+ancho+margen sea menor que la de la pelota se mueve solo hacia la derecha
    
    sub #5                      ;;MODIFICAR PARA RECUPERAR PELOTA
    cp  b
    jr  c, must_jump
    
    call sys_ia_stop
    ret

    must_jump:
    call sys_ia_recover_ball

    only_move_right:
    call sys_ia_move_right
    ret
    ;;Aqui veremos la y de la pelota para ver si debe saltar o no
    move_left:
    sub e_w(iy)
    sub #8
    cp b
    jr  nc, only_move_left 
    
    call sys_ia_jump

    only_move_left:
    call sys_ia_move_left
    
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SE DIRIGE HACIA LA IZQUIERDA 
;;
;;  INPUT:
;;  IX = puntero personaje ia
;;  IY = puntero al balon

sys_ia_move_left::

    ld  b, e_floor(ix)      ;;si no esta en su fila es que esta saltando
    ld  a, e_y(ix)          ;;metemos en b la y de la entidad
    cp  b
    ret nz                  ;; si no esta en el suelo mantiene la direccion 

    ld  e_vx(ix), #-1

    call man_entity_flip_left
ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SE DIRIGE HACIA LA DERECHA Y SALTA LA PELOTA PARA DIRIGIRSE A LA PORTERIA RIVAL 
;;
;;  INPUT:
;;  IX = puntero personaje ia
;;  IY = puntero al balon

sys_ia_move_right:: ;; aqui habra que poner que salte cuando llegue para ponerse al otro lado
    ld  e_vx(ix), #1

    call man_entity_flip_right
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  FUNCION PARA QUE SALTE LA IA
;; 
;;  INPUT:
;;  IX = puntero personaje ia
;;  IY = puntero al balon
;;
;; Veremos donde esta la pelota y saltara
;;
sys_ia_jump::

    ;;comprobamos la y de la pelota 
    ld  a, (ball_last_y)    ;; cogemos la last y de la pelota
    ld  b, a
    ld  a, e_floor(iy)      ;;cogemos la y maxima de la pelota
    sub #32
    cp  b                   ;;si da cero sera el mismo asi que no debe saltar la ia
    ret c

    sys_ia_recover_ball::   ;; Esta etiqueta sirve para reutilizar la funcion en el caso de salvar la bola en la derecha
    ld  b, e_floor(ix)      ;;si no esta en su fila es que esta saltando
    ld  a, e_y(ix)          ;;metemos en b la y de la entidad
    cp  b
    ret nz                  ;;si no es cero es porque ya esta saltando


    ld e_vy(ix), #-11
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; LA IA SE PARA
;;
;;  INPUT:
;;  IX = puntero personaje ia
;;  IY = puntero al balon

sys_ia_stop::
    ;;si la y del personaje mas su altura esta por encima de la pelota no se para
    ld  a, (ball_last_y)
    ld  b, a
    ld  a, e_y(ix)
    add e_h(ix)
    cp  b              
    ret c

    ld e_vx(ix), #0
ret