;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of 1to1 Soccer: An Amstrad CPC Game
;;  Copyright (C) 2020 Utopia (@UtopiaRetroDev)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;-------------------------------------------------------------------------------

;;
;; PHYSICS SYSTEM
;;
.module Physics_System


.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "man/entity.h.s"
.include "man/game.h.s"
.include "music_g.h.s"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; DATOS 
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

ball_time:: .db 0
ball_jump:: .db 0


;;array de teclas tipicas de amstrad
keyboard_player_1_op_2::
    .dw Key_Q, sys_physics_input_up             ;;arriba
    .dw Key_O, sys_physics_input_left           ;;izq
    .dw Key_P, sys_physics_input_right          ;;derecha
    .dw 0x0000

keyboard_player_2_op_2::
    .dw Joy0_Up, sys_physics_input_up           ;;arriba
    .dw Joy0_Left, sys_physics_input_left       ;;izq
    .dw Joy0_Right, sys_physics_input_right     ;;derecha
    .dw Joy0_Fire1, sys_physics_input_up        ;;arriba extra
    .dw 0x0000

;;array de teclas tipicas de ordenador
keyboard_player_1_op_1::
    .dw Key_W, sys_physics_input_up             ;;arriba
    .dw Key_A, sys_physics_input_left           ;;izq
    .dw Key_D, sys_physics_input_right          ;;derecha
    .dw 0x0000

keyboard_player_2_op_1::
    .dw Key_I, sys_physics_input_up             ;;arriba
    .dw Key_J, sys_physics_input_left           ;;izq
    .dw Key_L, sys_physics_input_right          ;;derecha
    .dw 0x0000


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion para comprobar si se ha pulsado alguna tecla del array 
;;
;; INPUT:
;;      IY: array de teclas a comprobar
;;      

sys_physics_keyboard_array::

nextkey:
    ld  bc, #4
    add iy, bc

    ;;Cogemos la siguiente tecla
    ld  l, (iy)
    ld  h, 1(iy)

    ;;vemos si hemos llegado ya al final del array
    ld  a, l
    or  h
    ret z               ;;nos salimos si es 0000

    ;;vemos si la tecla esta pulsada
    call cpct_isKeyPressed_asm
    jr  z, nextkey                      ;;si no esta pulsada pasamos a la siguiente

    ;;si la tecla esta pulsada, realizamos la accion
    ld  hl, #nextkey                    
    push hl                             ;;guardamos la dir a la que volver
    ld  l, 2(iy)
    ld  h, 3(iy)
    jp  (hl)                            ;;saltamos a la dir de mem de la accion asociada a la tecla


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion para comprobar si es jugador 1 o 2 y poner su array de teclas 
;;
;; INPUT:
;;      IX: puntero a la entidad a actualizar. Ix permite agregar desplazamientos
;;       

sys_physics_check_keyboard::

    ld  e_vx(ix), #0    ;;primero ponemos su vel a 0

    ;;vemos si la entidad es de tipo jugador 2 
    ld a, e_type(ix)
    and #e_type_player2
    jr nz, input_player2

array_key_player1 == . + 2
    ld  iy, #keyboard_player_1_op_2
    ld  bc, #-4
    add iy, bc
    jr  fin_check

    input_player2: 

array_key_player2 == . + 2
    ld  iy, #keyboard_player_2_op_2
    ld  bc, #-4
    add iy, bc

    fin_check:
    call sys_physics_keyboard_array

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion para moverse a la izq
;;
;; INPUT:
;;      IX: puntero a la entidad a actualizar. Ix permite agregar desplazamientos
;;   

sys_physics_input_left::

    ld  e_vx(ix), #-1

    call man_entity_flip_left

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion para moverse a la derecha
;;
;; INPUT:
;;      IX: puntero a la entidad a actualizar. Ix permite agregar desplazamientos
;;   

sys_physics_input_right::

    ld  e_vx(ix), #1

    call man_entity_flip_right

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion para moverse hacia arriba
;;
;; INPUT:
;;      IX: puntero a la entidad a actualizar. Ix permite agregar desplazamientos
;;   

sys_physics_input_up::

    ld  b, e_floor(ix)      ;;si no esta en su fila es que esta saltando
    ld  a, e_y(ix)          ;;metemos en b la y de la entidad
    cp  b
    ret  nz                 ;;si a!=b es que ya habia pulsado antes y no se hace

    ld  e_vy(ix), #-11      ;;ponemos la velocidad en y en menos uno ya que va hacia arriba
    
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion de actulizacion de una entidad
;;
;; INPUT:
;;      IX: puntero a la entidad a actualizar. Ix permite agregar desplazamientos
;;       

sys_physics_update_one_entity::
    
    ;;vemos si la entidad tiene el componente input 
    ld  a, e_cmp(ix)
    and #e_cmp_input

    call nz, sys_physics_check_keyboard     ;;si es input se comprueba si esta siendo pulsada

    ld  b, e_x(ix)          ;;cargo en a la x de la entidad
    ld  a, e_vx(ix)         ;;cargo en b su velocidad
    add a, b                ;;hago a+b 

    ;;comprobamos que no se salga de los limites del campo
    ld  b, #field_limit_left
    cp  b
    jr  c, out_of_limits

    ld  b, e_limit_r(ix)
    cp  b
    jr  nc, out_of_limits

    ld b, e_x(ix)
    ld e_last_x(ix), b
    ld e_x(ix), a       ;;actualizo la x de la entidad con el nuevo valor de a

    jr inside_of_limits

    out_of_limits:
    ;;comprobamos si es pelota para ver si es gol o fuera
    ld a, e_type(ix)
    and #e_type_ball
    jp  z, inside_of_limits  ;;si da 0 es que no es pelota

    ld  a, e_y(ix)
    ld  b, #field_limit_top
    cp  b
    jr  c, out_goal        ;;si la y del balon es mas alta que la de la porteria es que se ha salido

    ;;si es mas bajo marcamos con 1 que es gol
    ld a, #1
    ld (reset_goal), a  ;;lo ponemos a 1 para que en el bucle pase a resetear la pos y vel de las entidades

    ld  a, e_x(ix)
    ld  b, #field_center
    cp  b
    jr  c, player2

    ld  a, (goal_player1)   ;;incrementamos en 1 los goles del jugador 1
    inc a
    daa                     ;;para que de 9 pase a 0 y no a A
    ld (goal_player1), a

    ;;llamada para el sonido de gol de la der
    push ix                  ;;para no perderlo con la llamada
    call gol_der
    pop  ix

    ret

    player2:

    xor a                   ;;reseteamos el c para que no afecte al daa
    ld  a, (goal_player2)   ;;incrementamos en 1 los goles del jugador 2
    inc a
    daa                     ;;para que de 9 pase a 0 y no a A
    ld (goal_player2), a

    ;;llamada para el sonido de gol de la iz
    push ix                  ;;para no perderlo con la llamada
    call gol_izq
    pop  ix

    ret

    out_goal:

    ld  a, e_x(ix)
    ld  b, #field_center
    cp  b
    jr  nc, out_right

    ;;llamada para el sonido de fuera de la iz
    push ix                  ;;para no perderlo con la llamada
    call fuera_izq
    pop  ix

    ;;si se ha salido marcamos con 2 para que empiece en porteria izquierda
    ld a, #2
    ld (reset_goal), a  ;;lo ponemos a 1 para que en el bucle pase a resetear la pos y vel de las entidades

    ret

    out_right:

    ;;llamada para el sonido de fuera de la der
    push ix                  ;;para no perderlo con la llamada
    call fuera_der
    pop  ix

    ;;si se ha salido marcamos con 2 para que empiece en porteria derecha
    ld a, #3
    ld (reset_goal), a  ;;lo ponemos a 1 para que en el bucle pase a resetear la pos y vel de las entidades

    ret

    inside_of_limits:

    inc e_vy(ix)        ;;si es velocidad negativa le aumentamos el valor

    ld  b, e_y(ix)       ;;cargo en a la x de la entidad
    ld  a, e_vy(ix)      ;;cargo en b su velocidad
    add a, b            ;;hago a+b 
    
    ld e_y(ix), a     ;;actualizo la y de la entidad con el nuevo valor de a

    ld  b, e_floor(ix)     ;;si no esta en su fila es que esta saltando
    ld  a, e_y(ix)      ;;metemos en b la y de la entidad
    cp  b
    jr  c, fin_physics   ;;si B es mayor (el suelo) es que aun no ha llegado a su posicion

    ld  e_y(ix), b          ;;ponemos a la entidad en el suelo
    ld  e_vy(ix), #0        ;;velocidad en 0 para que no se mueva

    ;;comprobamos si es de tipo pelota para hacer acciones especiales de esta
    ld a, e_type(ix)
    and #e_type_ball
    call nz, sys_physics_update_ball

    fin_physics:
    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion de actulizacion del sistema fisico. 
;;      Utilizamos inversion de control para que sea el manager el que se encargue de solo pasar entidades validas
;;  
sys_physics_update::
    ld  de, #sys_physics_update_one_entity
    ld  hl, #func_update_forall             ;;modificamos la direccion de memoria en la que esta guardada la funcion a la que tiene que hacer call el update forall
    ld  (hl), e
    inc hl
    ld  (hl), d

    call man_entity_update_forall           ;;para hacer division de tareas y que manager se encargue de comprobar cada uno
    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion de actulizacion de la pelota.
;;      Vemos si tiene que rebotar o frenarse
;; 
;;  INPUT
;;      IX: pelota
;;

sys_physics_update_ball::

    ;;if ( y == floor_ball & ball_jump !=0 ) _rebota
    ld  a, e_y(ix)
    ld  b, #floor_ball
    cp  b
    ret  nz                 ;;si no es cero es que esta rebotando en este momento

    ld  a, (ball_jump)       ;;le sumamos dos al salto (ya que es negativo) para que llegue a 0
    or  a                   ;;lo comparamos con ella misma para ver si es 0
    jr  z, stop_ball        ;;si es 0 nos salimos
    
    add #2                  ;;si no es 0 le sumamos dos

    ld (ball_jump), a       ;;actualizamos ball_jum e v_y con el nuevo valor de a
    ld  e_vy(ix), a

    ret nz                  ;;si v_y no es 0 entonces es que tiene que rebotar y por tanto no se frena


    stop_ball:
    ;;if ( ball_time != 0 ) se decrementa
    ld  a, (ball_time)
    cp  b
    ret  z                  ;;si ball_time es 0 no hacemos los calculos de despues
      
    dec a                   ;;si v_y era 0 decrementamos el tiempo de la pelota en 1 cada vez que pase
    jr nz, fin_ball         ;;si no es 0 no modificamos el valor de vx

    ld e_vx(ix), a          ;;si es 0 metemos un 0 en v_x para que se pare

    fin_ball:
    ld (ball_time), a       ;;actualizamos el valor de ball_time con el nuevo valor de a

    ret