;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of 1to1 Soccer: An Amstrad CPC Game
;;  Copyright (C) 2020 Utopia (@UtopiaRetroDev)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;-------------------------------------------------------------------------------

;;
;; RENDER SYSTEM
;;
.module Render_System


.include "cpctelera.h.s"
.include "cpct_functions.h.s"
.include "man/entity.h.s"
.include "src/sprites/sprites.h.s"
.include "sys/render.h.s"
.include "man/game.h.s"
.include "man/scroll.h.s"
.include "sys/scroll.h.s"
.include "src/sprites/precalculated.h.s"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  DATOS PRIVADOS
;;

str_separator::      .asciz  ":"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion de inicializar el render
;;
;;   

sys_render_init::

    halt                                ;;para asegurarnos que no se ejecuta en medio de una interrup
    ld  c, #0                           ;;cargamos en c un 0 para que se ponga en el modo de video 0
    call cpct_setVideoMode_asm

    ;;colocamos en negro el borde de la pantalla
    cpctm_setBorder_asm  HW_BLACK  

    ld  hl, #_g_palette
    ld  de, #16
    call cpct_setPalette_asm     
    
    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion de renderizar de una entidad
;;
;; INPUT:
;;      IX: puntero a la entidad a renderizar. Ix permite agregar desplazamientos
;;       


sys_render_update_one_entity::


    ;;comprobamos si los jugadores estan dentro de la pantalla o no para el render
    ld  a, e_type(ix)
    and #(e_type_player1|e_type_player2)
    ld  a, e_x(ix)
    call nz, sys_scroll_check_within
    ret c                           ;;si la funcion nos devuelve el flag C es que no se ve en la pantalla

    ;;metemos en la llamada de abajo la dir de mem del dibujar de esa entidad
    ld  d, e_sprite_h(ix)
    ld  e, e_sprite_l(ix)

    ld  hl, #call_to_draw
    ld  (hl), e
    inc hl
    ld  (hl), d

    ;;Calcular la posicion en la memoria de video
    call sys_scroll_screen_start_DE  ;;relativos a la primera pos de pantalla

    call sys_scroll_calcule_x   ;;mete en c el valor de x de la entidad en funcion del comienzo de pantalla
    ld   b, e_y(ix)             ;;y -> carga en b lo que hay en ix+2
    call cpct_getScreenPtr_asm  ;;lo devuelve en hl  


call_to_draw = . + 1
    call #0x0000                ;;esta llamada se sustituira por la que tenga cada entidad

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion de render de todas las entidades 
;;      Utilizamos inversion de control para que sea el manager el que se encargue de solo pasar entidades validas
;;  

sys_render_update::

    ld  de, #sys_render_update_one_entity
    ld  hl, #func_update_forall             ;;modificamos la direccion de memoria en la que esta guardada la funcion a la que tiene que hacer call el update forall
    ld  (hl), e
    inc hl
    ld  (hl), d

    call man_entity_update_forall          ;;para hacer division de tareas y que manager se encargue de comprobar cada uno
    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion de render de la puntuacion y del tiempo la primera vez
;;

sys_render_scores_time_init::

    call sys_scroll_screen_start_DE
    ld  c, #4                           ;;x
    ld  b, #10                          ;;y
    call cpct_getScreenPtr_asm

    ;; pintamos en orden de pantalla

    ;;goles player 1 primer caracter
    ld  a, (goal_player1)
    rra                                 ;;rotamos los bits hacia la derecha
    rra
    rra
    rra                                 
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number

    ;;goles player 1 segundo caracter
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (goal_player1)                                
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number


    ;; Tiempo minutos
    ld  bc, #26                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (time_game+1)

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 


    ;;separador
    ld  bc, #4                         ;;desplazamos el puntero
    add hl, bc

    ld  a, #10

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ;; primer caracter segundos
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (time_game)
    rra                                 ;;rotamos los bits hacia la derecha
    rra
    rra
    rra                                 
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ;; segundo caracter segundos
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc


    ld  a, (time_game)                               
    and #0x0F                   ;; para quedarnos solo con la segunda parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 


    ;;goles player 2 primer caracter
    ld  bc, #22                        ;;desplazamos el puntero
    add hl, bc

    ld  a, (goal_player2)
    rra                                 ;;rotamos los bits hacia la derecha
    rra
    rra
    rra                                 
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 


    ;;goles player 2 segundo caracter
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (goal_player2)                                
    and #0x0F                           ;; para quedarnos solo con la primera parte


    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 
;; Funcion de render de la puntuacion y del tiempo
;;

sys_render_scores_time::

    ;;primero comporbamos que no queden tiles pendientes de hacer scoll
    ld  a, (scroll_offset)
    bit 7, a
    jr  nz, scroll_left 
    or  a                   ;;comprobamos que a no sea 0
    jp  nz, scroll_right
    jp  z, only_redraw_time ;;si es 0 es que no hay scroll y solo pintamos el tiempo (caracteres: el segundo del minuto y los dos del segundo)

    scroll_left::
    ;; pintamos en orden de pantalla
    xor   a
    ld    (scroll_offset), a


    call sys_scroll_screen_start_DE
    ld  c, #4                           ;;x
    ld  b, #10                          ;;y
    call cpct_getScreenPtr_asm

    ;;goles player 1 primer caracter
    ld  a, (goal_player1)
    rra                                 ;;rotamos los bits hacia la derecha
    rra
    rra
    rra                                 
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number

    ;;goles player 1 segundo caracter
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (goal_player1)                                
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number

    ;;goles player 1 segundo caracter para borrarlo
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    call sys_render_erase_scores

    ;; Tiempo minutos
    ld  bc, #22                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (time_game+1)

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 


    ;;separador
    ld  bc, #4                         ;;desplazamos el puntero
    add hl, bc

    ld  a, #10

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ;; primer caracter segundos
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (time_game)
    rra                                 ;;rotamos los bits hacia la derecha
    rra
    rra
    rra                                 
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ;; segundo caracter segundos
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc


    ld  a, (time_game)                               
    and #0x0F                   ;; para quedarnos solo con la segunda parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ;; segundo caracter segundos para borrar
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    call sys_render_erase_scores

    ;;goles player 2 primer caracter
    ld  bc, #18                        ;;desplazamos el puntero
    add hl, bc

    ld  a, (goal_player2)
    rra                                 ;;rotamos los bits hacia la derecha
    rra
    rra
    rra                                 
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 


    ;;goles player 2 segundo caracter
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (goal_player2)                                
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ;;goles player 2 segundo caracter para borrar
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    call sys_render_erase_scores

    ret

    scroll_right:

    ;; pintamos en orden de pantalla
    xor   a
    ld    (scroll_offset), a


    call sys_scroll_screen_start_DE
    ld  c, #0                           ;;x
    ld  b, #10                          ;;y
    call cpct_getScreenPtr_asm

    ;;goles player 1 primer caracter para borrarlo
    call sys_render_erase_scores

    ;;goles player 1 primer caracter
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (goal_player1)
    rra                                 ;;rotamos los bits hacia la derecha
    rra
    rra
    rra                                 
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number

    ;;goles player 1 segundo caracter
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (goal_player1)                                
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number


    ;; caracter minutos para borrar
    ld  bc, #22                          ;;desplazamos el puntero
    add hl, bc

    call sys_render_erase_scores

    ;; Tiempo minutos
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (time_game+1)

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 


    ;;separador
    ld  bc, #4                         ;;desplazamos el puntero
    add hl, bc

    ld  a, #10

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ;; primer caracter segundos
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (time_game)
    rra                                 ;;rotamos los bits hacia la derecha
    rra
    rra
    rra                                 
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ;; segundo caracter segundos
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc


    ld  a, (time_game)                               
    and #0x0F                   ;; para quedarnos solo con la segunda parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 


    ;;goles player 2 primer caracter para borrar
    ld  bc, #18                          ;;desplazamos el puntero
    add hl, bc

    call sys_render_erase_scores

    ;;goles player 2 primer caracter
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (goal_player2)
    rra                                 ;;rotamos los bits hacia la derecha
    rra
    rra
    rra                                 
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 


    ;;goles player 2 segundo caracter
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (goal_player2)                                
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 
    
    ret

    only_redraw_time:
    
    ld  a, (time_change)
    or  a
    ret z

    xor a
    ld  (time_change), a

    call sys_scroll_screen_start_DE
    ld  c, #34                           ;;x
    ld  b, #10                          ;;y
    call cpct_getScreenPtr_asm

    ;; Tiempo minutos
    ld  a, (time_game+1)

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ;; primer caracter segundos
    ld  bc, #8                          ;;desplazamos el puntero
    add hl, bc

    ld  a, (time_game)
    rra                                 ;;rotamos los bits hacia la derecha
    rra
    rra
    rra                                 
    and #0x0F                           ;; para quedarnos solo con la primera parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ;; segundo caracter segundos
    ld  bc, #4                          ;;desplazamos el puntero
    add hl, bc


    ld  a, (time_game)                               
    and #0x0F                   ;; para quedarnos solo con la segunda parte

    ;;calcula la dir de memoria de la funcion de pintar y lo pinta
    call sys_render_draw_number 

    ret


sys_render_erase_scores::
    ld  a, #11

    call sys_render_draw_number

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;  FUNCION QUE CALCULA LA DIR DE MEM DEL NUMERO A DIBUJAR Y LO PINTA
;;      INPUT
;;          -A: numero a sumar a la dir de mem
;;

sys_render_draw_number::
    push hl

    ld  hl, #tileset_numbers
    sla a                               ;;multiplicamos por 2
    add_hl_a                            ;;le sumamos a hl el valor de a dos ves porque la dir de mem es de tamny 2
    ld  e, (hl)
    inc hl
    ld  d, (hl)

    ld  hl, #call_draw_number
    ld  (hl), e
    inc hl
    ld  (hl), d

    pop  hl
    push hl

call_draw_number = . + 1
    call    #0x0000                     ;;esto se cambia con la dir de mem sacada de antes
    
    pop  hl

    ret


;; comprobar si es izq o derecha y crear tres recorridos

;;derecha
;; call erase
;; call draw
;; call draw

;;etc

;;izquierda
;; call draw
;; call draw
;; call erase

;; etc

;;zero
;;solo repintammos el tiempo en la misma pos



