;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of 1to1 Soccer: An Amstrad CPC Game
;;  Copyright (C) 2020 Utopia (@UtopiaRetroDev)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see http://www.gnu.org/licenses/.
;;-------------------------------------------------------------------------------

;;
;; SCROLL SYSTEM
;;

.module Scroll_System

.include "cpctelera.h.s" 
.include "cpct_functions.h.s"
.include "man/scroll.h.s"
.include "man/entity.h.s"
.include "src/sprites/sprites.h.s"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; DATOS
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;los usamos para redibujar el fondo con clipping
tw_c::   .db 0
tx_c::   .db 0
tile_c:: .db 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCIONES
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; funcion izquierda para decrementar los tiles que quedan por mover y poner el offset
;;

sys_scroll_left::
   ;;decrementamos el tiles to scroll
   inc (hl)                      ;;como va hacia la izquierda es neg, por lo que para decrementar hay que sumar

   ld  a, #-1
   ld  (scroll_offset), a
   ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; funcion derecha para decrementar los tiles que quedan por mover y poner el offset
;;

sys_scroll_right::
   ;;decrementamos el tiles to scroll
   dec (hl)          

   ld  a, #1
   ld  (scroll_offset), a           
   ret 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; funcion para poner los tiles que nos tenemos que mover
;;

sys_scroll_start_right::
   ld  a, #10
   .db #0xD2                  ;;salto si hay nc a la linea 57, pero como sabemos que entra con c se lo salta y se va a la linea de abajo
sys_scroll_start_left::
   ld  a, #-10
   ld  (tiles_to_scroll), a

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; funcion para controlar la pelota para hacer el scroll
;;

sys_scroll_ball::
   ;;primero comporbamos que no queden tiles pendientes de hacer scoll
   ld  hl, #tiles_to_scroll
   ld  a, (hl)
   bit 7, a
   jr  nz, sys_scroll_left 
   or  a                   ;;comprobamos que a no sea 0
   jr  nz, sys_scroll_right

   ;;Detectamos el nuevo scroll en el caso de que tiles_to_scroll sea 0
   ;; guardamos en IX la pelota
   call man_entity_getArrayHL

   ex de, hl

   ld__ixl_e               ;;guardamos DE en IX
   ld__ixh_d
    
   ld iy, #hwscroll        ;;metemos en iy hwscroll

   
   ld  a, hs_pVideo_l(iy)  ;;posicion actual de la mem de video
   add #10                 ;;le sumamos para poner un margen
   ld  b, a                ;;la pasamos a b
   ld  a, e_x(ix)          ;;cargamos su x en a

   cp  b

   jp  c, scroll_ball_left      ;;si a es menor que b es que esta fuera por la izq asi que nos salimos             
    
   ld  b, a                ;;pasamos la x del jugador a b
   ld  a, hs_pVideo_l(iy)  ;;metemos en a la pos de la mem de video
   add #69                 ;;le sumamos para llegar al margen derecho 

   cp  b                   ;;si a es mayor que b es que esta dentro por la derecha

   ret nc                  ;;si a es mayor que b es que esta dentro por la der asi que nos salimos 
   
   call sys_scroll_start_right

   ret

   scroll_ball_left:

   call sys_scroll_start_left

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA DEC O INC HL EN FUNCION DE SCROLL_OFFSET
;;

change_hl:

operation = .           ;;apunta a inc, para que algunas veces haga inc y otras dec
   inc  hl

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION DE SCROLL
;;    Actualizamos los punteros de hwscroll
;;       INPUT:
;;          IX: dir de memoria donde se almacena lo del scroll
;;

sys_scroll_tilemap::
   ld   a, (scroll_offset)      ;;comprobamos si scroll es 1 o -1
   ld   hl, #operation          ;;cargamos en hl la dir de mem de operation para que dependiendo del valor de scroll haga inc o dec
   dec  a
   jp   z, update_column
    
   ld   (hl), #0x2B             ;;si scroll es -1 hacemos dec de hl

   xor  a                      ;;ponemos a 0 la columna
   ld   (column_scroll), a
   jp   update_scroll

   update_column:
   ld   (hl), #0x23             ;;si scroll es 1 hacemos inc de hl
   ld   a, #scr_tile_width
   dec  a
   ld   (column_scroll), a      ;;ponemos la columna en la ultima de la pantalla
   
   update_scroll:

   ld   l, hs_pVideo_l(ix)      ;;actualizamos el puntero a la memoria de video para que se mueva 4 para el lado
   ld   h, hs_pVideo_h(ix)
   call change_hl             ;;para que haga dec o inc en funcion del scroll
   call change_hl             ;;son 4 porque es de 4x8
   call change_hl
   call change_hl
   ld   hs_pVideo_l(ix), l
   ld   hs_pVideo_h(ix), h

   ld   l, hs_pTilemap_l(ix)    ;;actualixamos el puntero al tilemap para que se mueva uno para el lado
   ld   h, hs_pTilemap_h(ix)
   call change_hl
   ld   hs_pTilemap_l(ix), l
   ld   hs_pTilemap_h(ix), h

   ld   a, (scroll_offset)
   sla  a                      ;;lo multiplicamos por dos
   ld   c, a
   ld   a, hs_scroll(ix)
   add  c
   ld   hs_scroll(ix), a        ;;actualizamos el scroll de hwscroll con el scroll offset
ret


sys_scroll_draw::

   ld  ix, #hwscroll        ;;cogemos la dir de mem de donde estan los valores del scroll
   ld   l, hs_scroll(ix)
   call cpct_setVideoMemoryOffset_asm  ;;cambiamos el offset de la memoria (donde empieza lo que se ve)

   ;; Pintamos la nueva columna de la pantalla
   ld   l, hs_pVideo_l(ix)
   ld   h, hs_pVideo_h(ix)          ;; HL = puntero a la memoria de video 
   ex   de, hl
   ld   a, (column_scroll)
   sla  a                         ;;multiplico por 2
   sla  a                         ;;multiplico por 2
   add_de_a

   ld   l, hs_pTilemap_l(ix)
   ld   h, hs_pTilemap_h(ix)       ;; DE = puntero al tilemap
   ex   de, hl
   ld   a, (column_scroll)
   add_de_a

   call cpct_etm_drawTilemap4x8_ag_asm ;;pintamos la nueva columna

   ld   ix, #hwscroll
   ld   a, (scroll_offset)
   sub  #1
   jp   z, update_top_left

   ld   h, hs_pVideo_h(ix)
   ld   l, hs_pVideo_l(ix)
   ex   de, hl
   ld   c, #0             ;;x
   ld   b, #8*map_height  ;;y
   call cpct_getScreenPtr_asm

   ex   de, hl         ;;dir memoria
   ld   hl, #_black_tile

   call cpct_drawTileZigZagGrayCode4x8_af_asm

   ret

   update_top_left:

   ld   h, hs_pVideo_h(ix)
   ld   l, hs_pVideo_l(ix)
   dec  hl
   dec  hl
   dec  hl
   dec  hl
   ex   de, hl         ;;dir memoria
   ld   hl, #_black_tile

   call cpct_drawTileZigZagGrayCode4x8_af_asm
   
   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION UPDATE SCROLL
;;

sys_scroll_update::

    ;;Ponemos el tileset preparado para luego (AHORA SOLO ES 1 DE ANCHO)
    ld   c, #1
    ld   b, #map_height
    ld   de, #map_width
    ld   hl, #_tiles_00
    call cpct_etm_setDrawTilemap4x8_ag_asm

    call sys_scroll_ball ;;nos devuelve en a el valor a donde mover

    ld  ix, #hwscroll        ;;cogemos la dir de mem de donde estan los valores del scroll

    ld  a, (scroll_offset)
    or  a
    ret z                    ;;si scroll_offset es 0 es que no se ha desplazado y nos salimos

    ld  a, (scroll_offset)
    dec a 
    jr  nz, continue_left

    ld  a, hs_scroll(ix)     ;;cogemos su scroll actual
    ld  b, #max_scroll       ;;lo comparamos con el scroll maximo
    cp  b
    ret z              ;;si son iguales nos salimos sin hacer nada
    jp  nz, move_scroll

    continue_left:
    ld  a, hs_scroll(ix)     ;;cogemos su scroll actual
    or  a                    ;;hacemos un or para ver si da 0
    ret z             ;;si son iguales nos salimos sin hacer nada

    move_scroll:
    call sys_scroll_tilemap   ;;en caso de moverse actualizamos punteros y pintamos las nuevas columnas

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA COMPROBAR SI EL JUGADOR ESTA DENTRO DE LA PARTE DE 
;;  LA PANTALLA QUE SE ESTA MOSTRANDO
;;          INPUT:
;;              -A: valor de x a comprobar
;;          OUTPUT:
;;              -FLAG C si no esta dentro de lo que se muestra
;;

sys_scroll_check_within::

   ld iy, #hwscroll        ;;metemos en iy hwscroll

   ;ld  a, e_x(ix)          ;;cargamos su x en a
   ld  b, hs_pVideo_l(iy)  ;;posicion actual de la mem de video

   cp  b

   ret c                   ;;si a es menor que b es que esta fuera por la izq              
   
   ld  b, a                ;;pasamos la x del jugador a b
   ld  a, hs_pVideo_l(iy)  ;;metemos en a la pos de la mem de video

   add #70                 ;;le sumamos 69 para llegar al margen derecho (79-9 del ancho del player)

   cp  b                   ;;si a es mayor que b es que esta dentro por la derecha

   ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA COMPROBAR SI EL JUGADOR ESTA DENTRO DE LA PARTE DE 
;;  LA PANTALLA QUE SE ESTA MOSTRANDO
;;          INPUT:
;;              -IX: dir de memoria del jugador
;;          OUTPUT:
;;              -C: posicion en X de la entidad en funcion del inicio de la pantalla
;;

sys_scroll_calcule_x::

   ld iy, #hwscroll        ;;metemos en iy hwscroll

   ld  a, e_x(ix)          ;;cogemos la x de la entidad
   sub hs_pVideo_l(iy)     ;;le restamos donde comienza la memoria de video

   ld  c, a                ;;lo guardamos en c para que le sirva a la funcion que lo llama

   ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA METER EN DE LA PRIMERA POS DE LA PANTALLA 
;;

sys_scroll_screen_start_DE::

   ld iy, #hwscroll        ;;metemos en iy hwscroll

   ld d, hs_pVideo_h(iy)
   ld e, hs_pVideo_l(iy)

ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA RESTAURAR EL TILEMAP DONDE ESTABAN LOS PERSONAJES 
;;

sys_scroll_restore_background::
   
   call man_entity_getArrayHL    ;;metemos en hl el inicio del array de entidades

   ex de, hl                     ;;lo pasamos a DE

   ld__ixl_e                     ;;guardamos DE en IX
   ld__ixh_d

   restore_background:
   ld  a, e_type(ix)                ;;cojo su tipo
   ld  b, #e_type_invalid           ;;guardo en b el tipo de entidad invalido

   cp  b                            ;;comparo a con b
   ret  z                           ;;si da cero es que es invalida y hemos acabado de actualizar

   ;;ponemos los valores de tw_c, tx_c, tile_c como si no hubiera clipping
   ld a, e_tw(ix)
   ld (tw_c), a

   ld a, e_tx(ix)
   ld (tx_c), a

   xor a
   ld (tile_c), a

   ;;comprobamos que la posicion anterior este en la pantalla
   ld   a, e_last_x(ix)                   ;;la x anterior del conjunto de tiles a borrar
   call sys_scroll_check_within
   jr  c, next_entity                     ;;si devuelve c es que no esta dentro de la pantalla

   ;;AQUI DENTRO PROCESO DE RESTAURACION DEL TILEMAP
   ;;Ponemos el tileset preparado para el tamanyo de cada sprite a borrar
   ld   a, (tw_c)
   ld   c, a
   ld   b, e_th(ix)
   ld   de, #map_width
   ld   hl, #_tiles_00
   call cpct_etm_setDrawTilemap4x8_ag_asm
   
   push ix

   ;;Calcular la posicion en la memoria de video
   call sys_scroll_screen_start_DE  ;;relativos a la primera pos de pantalla

   ld   iy, #hwscroll         ;;metemos en iy hwscroll

   ld   a, (tx_c)
   sub  hs_pVideo_l(iy)       ;;le restamos donde comienza la memoria de video (para convertir la x a tamny de pantalla)
   ld   c, a
   ld   a, e_ty(ix)  
   ld   b, a    
   call cpct_getScreenPtr_asm

   ld   d, e_tilePos_h(ix)
   ld   e, e_tilePos_l(ix)
   ld   a, (tile_c)
   add_de_a

   call cpct_etm_drawTilemap4x8_ag_asm ;;pintamos la nueva columna
   
   pop ix                           ;;recuperamos ix
   
   next_entity:                   
   ld  bc, #entity_size             ;;meto en bc lo que ocupa una entidad
   add ix, bc                       ;;se lo sumo a ix para pasar a la siguiente
   jr  restore_background           ;;volvemos a repetir el proceso hasta que se salga por una invalida


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; FUNCION PARA recalcular posiciones del tile en los personajes 
;;

sys_scroll_recalculate_tile::
   
   call man_entity_getArrayHL    ;;metemos en hl el inicio del array de entidades

   ex de, hl                     ;;lo pasamos a DE

   ld__ixl_e                     ;;guardamos DE en IX
   ld__ixh_d

   recalculate_background:
   ld  a, e_type(ix)                ;;cojo su tipo
   ld  b, #e_type_invalid           ;;guardo en b el tipo de entidad invalido

   cp  b                            ;;comparo a con b
   ret  z                           ;;si da cero es que es invalida y hemos acabado de actualizar

   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;;actualizamos los valores del scroll en funcion de la nueva x
   ;; TX = 4*((int)x/4)
   ;;lo hacemos asi para que nos sirva para calcular el getScreenPtr en el scroll

   ld   a, e_x(ix)       ;;cargo en a el valor de x
   and  #0xFC
   ld   b, a           ;;lo metemos en b para ahora comparar el nuevo tx

   ld   a, e_tx(ix)            ;;cargamos en a el tx viejo
   cp   b                      ;;lo comparamos con el tx nuevo
   jr   z, continue_y          ;;si es el mismo no hacemos nada

   ld   e_tx(ix), b            ;;al no ser la misma actualizamos tx
   ld   h, e_tilePos_h(ix)     ;;si no es 0 guardamos la dir de mem del tile en hl para actualizarla (el jump siguiente es por la operacion anterior no por esto)
   ld   l, e_tilePos_l(ix)
          
   jr   nc, tile_move_left     ;;el nuevo tx es menor que el anterior

   inc  hl                     ;;irse a la der es incrementar en 1
   ld   e_tilePos_h(ix), h     ;;actualizamos el valor anterior de tilePos
   ld   e_tilePos_l(ix), l    

   jr continue_y
    
   tile_move_left:

   dec  hl                     ;;irse a la izq es decrementar en 1
   ld   e_tilePos_h(ix), h     ;;actualizamos el valor anterior de tilePos
   ld   e_tilePos_l(ix), l 
   ;;
   ;;fin actualizar x del scroll
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
   continue_y:
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   ;; actualizamos los parametros del scroll en funcion de la nueva y
   ;; TY = 8*((int)y/8)
   ld   a, e_y(ix)
   and  #0xF8
   ld   b, a           ;;lo metemos en b para ahora comparar el nuevo ty

   ld   a, e_ty(ix)                     ;;metemos el ty viejo en a
   cp   b                               ;;lo comparamos con el ty nuevo
   jr   z, next_entity_calculate        ;;si es el mismo no hacemos nada

   ld   e_ty(ix), b            ;;al no ser la misma actualizamos ty
   ld   h, e_tilePos_h(ix)     ;;si no es 0 guardamos la dir de mem del tile en hl para actualizarla (el jump siguiente es por la operacion anterior no por esto)
   ld   l, e_tilePos_l(ix)

   jr   nc, tile_move_up       ;;el nuevo ty es menor que el anterior

   ld   b, a                   ;;cargamos en b el valor del viejo ty
   ld   a, e_ty(ix)            ;;cargamos en a el nuevo ty
   sub  b                      ;;le restamos al nuevo (que es mas grande) el viejo

   srl  a  ;;|
   srl  a  ;;| -> dividimos entre 8 para tener el numero en tiles
   srl  a  ;;|

   ld   bc, #60                ;;irse hacia abajo es incrementar en 60

   loop_positive:
      add  hl, bc
      dec  a
   jr  nz, loop_positive

   ld   e_tilePos_h(ix), h     ;;actualizamos el valor anterior de tilePos
   ld   e_tilePos_l(ix), l    

   jr  next_entity_calculate
    
   tile_move_up:

   sub  b                      ;;le restamos al viejo (que es mas grande) el nuevo
   srl  a  ;;|
   srl  a  ;;| -> dividimos entre 8 para tener el numero en tiles
   srl  a  ;;|

   ld   bc, #-60                ;;irse hacia abajo es incrementar en 60

   loop_negative:
      add  hl, bc
      dec  a
   jr  nz, loop_negative

   ld   e_tilePos_h(ix), h     ;;actualizamos el valor anterior de tilePos
   ld   e_tilePos_l(ix), l 

   ;;fin actualizar scroll por parte de la y
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   
   next_entity_calculate:              
   ld  bc, #entity_size             ;;meto en bc lo que ocupa una entidad
   add ix, bc                       ;;se lo sumo a ix para pasar a la siguiente
   jr  recalculate_background       ;;volvemos a repetir el proceso hasta que se salga por una invalida


